/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *     "Apache Jetspeed" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache" or
 *    "Apache Jetspeed", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.jetspeed.portlets;

//Java stuff
import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;
import java.io.IOException;


//jetspeed stuff
import org.apache.jetspeed.portlets.util.SAXPIFilter;
import org.apache.jetspeed.portlet.*;



/**
Portlet to output an XML well-formed document without any rendering.
Strips all PIs and XML declaration before outputting content to allow
for inclusion in a wider XML document.

@author <A HREF="mailto:raphael@apache.org">Raphal Luta</A>
@version $Id: XMLPortlet.java,v 1.19 2001/03/07 06:47:17 taylor Exp $
*/
public class XMLPortlet extends AbstractPortlet {



    public void doView (PortletRequest request,
                         PortletResponse response) throws PortletException,
                                                          IOException
    {
        if (this.getPortletLog().isDebugEnabled())
	{
	    this.getPortletLog().debug("XMLPortlet:Entering doView");
	}
	
        try 
	{
	    ByteArrayOutputStream bos= new ByteArrayOutputStream();

	    if (this.getPortletLog().isDebugEnabled())
	    {
		this.getPortletLog().debug("XMLPortlet:Filtering Content");
	    }
            new SAXPIFilter(new PrintWriter(bos),true).print(  
		    this.getPortletConfig().getContext().getResourceAsStream(
			    request.getPortletSettings().getAttribute("url")));

	    if (this.getPortletLog().isDebugEnabled())
	    {
		this.getPortletLog().debug("XMLPortlet:writing output");
	    }
            response.getWriter().println(bos.toString());

        }
	catch (java.io.UnsupportedEncodingException exc)
	{
	    this.getPortletLog().error("XMLPortlet:Unable to encode url!");
	    throw new UnavailableException("XMLPortlet:Unable to encode url!"+exc.getMessage());
	}
    }
}
